// Game.h: interface for the Game class.
//
//////////////////////////////////////////////////////////////////////

/**
 * documentation ralis par Michel de VERDELHAN : michel.deverdelhan@free.fr
 * le 20/10/2004
 */

#ifndef _GAME_H_
#define _GAME_H_


#ifdef WIN32
#include <windows.h>
#endif
#include <GL/GL.h>
#include "GLUT.h"
#include "GL/glut.h"
#include "ModelDrawer.h"
#include "BasicRender.h"
#include "EnhancedRender.h"
#include "FastRender.h"
#include "Frustum.h"
#include "LoutreManager.h"
#include "Immeuble.h"
#include "ARB_Multitexture.h"
#include "ARB_Vertex_program.h"
#include "SkyBox.h"
#include "LastMenu.h"
#include "Font.h"
#include "MotionBlur.h"
#include "MainMenu.h"
#include "CollisionManager.h"
#include "Glow.h"
#include "EndMenu.h"

#include "Joueur.h"



#define RADAR_MIDDLE_X					360			//position sur X du radar
#define RADAR_MIDDLE_Z					40			//position sur Y du radar
#define RADAR_SIZE						32			//la largeur du radar/2
#define RADAR_REAL_LIMIT				1000		//la limite du radar dans le monde 3d
#define RADAR_ECHELLE					31.25		//l'echelle RADAR_REAL_LIMIT/RADAR_SIZE

//TODO regler le rayon
#define RADAR_POINTSIZE					1			//la largeur d'un point du radar

#define RACINE22						1.414		//rsq(2)			


#define DISTANCE_MAX_SHOOT				1500.0		//port du fusil pour repousse la loutre


//Game implemente un paterne singleton
/**
 * La classe game est la classe principale du jeu.
 * </BR>En effet c'est elle qui implemente la boucle principale de rendu du jeu (par contre
 * elle ne prend pas en compte les menu), la gestions des entre/sortie pendant le jeu (clavier, souris...)
 * c'est cette classe qui contient le manager de loutre, le manager de collision et certaine texure partage
 * entre plusieurs objets.
 * </BR>Elle gere aussi le radar en bas de l'ecran permettant de voir les loutres s'approchants de l'immeuble
 * et le message affich lorsque les point de vie restants sont trop bas.
 * </BR>La methode draw gere tout les affichage du jeu : 
 * <LI>les loutres en faisant appel a la methode LoutreManager::manageLoutres(int) qui affiche l'ensemble des loutres
 * du jeu.
 * <LI>l'immeuble en l'affichant simplement
 * <LI>Le joueur en faisant appel a sa methode draw
 * <LI>Le sol avec l'effet d'eclairage
 * <LI>Les limites du decors.
 * <LI>La sky box
 * </BR>C'est aussi cette classe qui gere la detection des tirs sur les loutre avec la methode shooting().
 * </BR>Cette classe untilise le pattern <B>SINGLETON</B> pour s'assurer de n'avoir qu'une et une seule
 * instance possible. Ceci nous oblige  avoir une methode startGame() qui initialise une nouvelle partie.
 * </BR>on peut aussi noter la methode loadRessources qui recharge l'ensemble des ressources modifiable.
 * les ressources modifiable sont les rextures, model et autres dont les parametres de qualite peuvent etres chang 
 * dans les differents menu du jeu.
 */
class Game : public GLUT  
{
private:

	/**
	 * permet de savoir si on utilise le model basse resolution de la loutre
	 */
	bool useLoutreLow;

	/**
	 * permet de savoir si le jeu est en pause
	 */
	bool paused;

	/**
	 * classe gerant le motion blur
	 */
	MotionBlur * motionBlur;

	/**
	 * variable deffinisant les parametres du message "warning" affich au cours du jeu
	 */
	float warningFactor[3];
	float warningValue[3];

	/**
	 * le rayon limite de la zone de jeu.
	 */
	int limiteRadius;

	/**
	 * le niveau de difficult de la partie
	 */
	int difficulty;

	/**
	 * le nombre de loutres du loutre manager
	 */
	int loutreNumber;

	/**
	 * la sky box affich autoure de l'environnement
	 */
	SkyBox * skyBox;

	/**
	 * Le model de l'immeuble
	 */
	Model * immeubleModel;

	/**
	 * le rayon de la lumiere tournant autour de l'immeuble
	 */
	float lightRadius;

	/**
	 * la texture de lightMap utiliser pour simuler l'effet de lumiere sur le sol
	 */
	RULES_Texture * lightMap;

	/**
	 * la texture utiliser pour le toon shading.
	 * la texture de toon shading est en fait une texture en degrad de gris mais avec 
	 * un contraste entre les niveau de gris tres important. utilis avec de l'eclairage par vertex selon
	 * la normal du vertex, on obtient l'effet de "coloriage" par zone du toon shading.
	 */
	RULES_Texture * toon;

	/**
	 * la texture utilise pour le glow.
	 * pour effectuer le glow, on commence par rendre la scene (seulement les partie que l'on souhaite voir
	 * briller) dans un petit viewport, on recupere l'image qu'on
	 * stock dans cette texture, puis on effectue du blurring sur cette texture, et au final, on ajoute cette texture
	 * a l'image final.
	 */
	RULES_Texture * toon_glow;

	/**
	 * la texture utilise pour le sol.
	 */
	RULES_Texture * sol;

	/**
	 * le constructeur par defaut de game.
	 * Ce constructeur est mis en private afin d'empecher toute classe ou fonction exterieur de cre une
	 * instance de Game autre que l'instance unique du pattern <B>SINGLETON</B>
	 */
	Game();

	/**
	 * Le model Drawer.
	 * Le systeme de Render path repose entierement sur le systeme des ModelDrawer.
	 * chaque render path a un ModelDrawer qui lui est attribu. Pour plus de renseignement sur les render
	 * path, voir les classes BasicRender , EnhencedRender et FastRender .
	 * </BR>Le type de renderer utilis est determin lors de la cration de l'instance de Game en regardant les
	 * differentes extentions presentes sur la carte graphique. Par la suite, lors du rendu, chaque model est envoy
	 * au renderer qui les affiche en fonction du render path. Cette methode permet de pouvoir
	 * avoir autant de methode de rendu que l'on veut tres facilement, il suffit de cre une nouvelle
	 * classe heritant de ModelDrawer et de rajouter sa detection dans le constructeur de la classe.
	 */
	ModelDrawer * drawer;

	/**
	 * Le joueur.
	 */
	Joueur *joueur;		//le joueur-camera

	/**
	 * la position de la lumiere qui tourne autour de l'immeuble
	 */
	RULES_Vector * lightPos;

	/**
	 * Le Loutre manager.
	 * c'est le loutre manager qui a en charge l'ensemble de la gestion des loutre. Il les fait se deplacer,
	 * les affiches, et gere leur reinitialisation.
	 */
	LoutreManager * loutreManager;

	/**
	 * Le model de la loutre. Ce model peut etre soit en haute resolution soir en basse resolution.
	 */
	Model * loutreModel;

	/**
	 * Le quadrique GLU qui delimite le terrain de jeu
	 */
	GLUquadricObj *limite;// le quadrique qui limite le terrain 

	/**
	 * La fonte utilise pour aficher les information pendant le jeu (vie, warning...)
	 */
	Font * font;

	/**
	 * La texture de la font.
	 */
	RULES_Texture  * texFont;

	/**
	 * Le gestionnaire de collisions. Gere les collision loutre/loutre, les collision loutre/joueur et joueur/immeuble
	 * par contre les collision joueur/limite du terrain sont gere dans la methode idle() de Game
	 */
	CollisionManager * collisionManager;

	/**
	 * permet de savoir si le joueur vient juste de tirer, 
	 */
	bool hasJustFired;

	/**
	 * Le frustum (cone) de vision du joueur
	 */
	Frustum *frustum;

	/**
	 * La position de la souris en coordonne ecran
	 */
	int mouseX,mouseY;

	/**
	 * temps de depart utilis pour calculer le score (une partie du score est du au temps ecoul dans le jeu,
	 * plus le joueur tiens longtemps, plus il gagne de points).
	 */
	int startTimeForScore;

	/**
	 * le nombre de loutres tue.
	 */
	int nbKilledLoutreForScore;

	/**
	 * Le nombre de tir du joueur
	 */
	int nbShoot;

	/**
	 * Le nombre de tirs ayants atteins une loutre.
	 */
	int nbGoodShoot;
	
	/**
	 * La texture du radar
	 */
	RULES_Texture * hud_radar;

	/**
	 * La texture d'un point sur le radar
	 */
	RULES_Texture * hud_rpoint;

	/**
	 * La texture du balayeur du radar
	 */
	RULES_Texture * hud_rbalayeur;

	/**
	 * l'angle de rotation du balayeur du radar
	 */
	float hud_balayeur_alpha;		//angle de rotation du balayeur

	/**
	 * la rapidit du balayeur du radar
	 */
	float hud_balayeur_deltaAlpha;	//rapidit du balayeur

	/**
	 * La classe s'occupant de la gestion du glow ambiant
	 */
	Glow *glow;

	/**
	 * Le niveau de glow
	 */
	unsigned int glowLevel;

	/**
	 * Methode utilise pour afficher le radar
	 */
	void renderRadar();

protected:

	/* nothing need to be protected */

public:
	
	/**
	 * definit le niveau de glow
	 * @param glowL: le niveau de glow
	 */
	void setGlowLevel(unsigned int glowL);

	/**
	 * recharge les differentes ressources utilise par le jeu qui sont suceptibles de changer en fonction du 
	 * niveau de qualit selectionn par l'utilisateur. Les ressource recharge sont les textures des
	 * models, du sol, de la skyBox et le model des loutres.
	 * @param texQuality: la qualite des texture: 0 = qualite original, 1 = taille des textures / 2, 2 = taille/ 4, 3 = taille / 8 etc...
	 * @param filtrageQality: qualit du filtrage
	 * <LI> defaut : pas de filtrage (GL_NEAREST)
	 * <LI> 1 : lineaire (GL_NEAREST)
	 * <LI> 2 : bilineaire (GL_LINEAR_MIPMAP_NEAREST)
	 * <LI> 3 : trilineaire (GL_LINEAR_MIP_MAP_LINEAR)
	 * @param loutreLow: utilise le model de la loutre basse resolution.
	 */
	void loadRessources(int texQuality, int filtrageQuality, bool loutreLow);

	/**
	 * retourne un pointeur sur le joueur
	 * @return Joueur*: le joueur
	 */
	Joueur * getPlayer();

	/**
	 * met en pause le jeu/enleve la pause
	 * @param pause: true => met le jeu en pose
	 */
	void setPaused(bool pause);

	/**
	 * effectue les test lors d'un tir (verifie si une loutre est touche, si elle est touche, lui
	 * applique les force...)
	 */
	void shooting();

	/**
	 * remet le temps ecoul a 0, utilise lorsque le joueur quitte une partie pour aller dans la menu, si cette
	 * methode n'est pas appel lorsqu'il revient dans le jeu, il y a une grande difference de temps entre les
	 * 2 image (celle aant d'aller dans le menu et celle quand on revint), ce qui poserait des probleme (loutre se deplacant
	 * trop, problemes de collisions...)
	 */
	void resetTime();

	/**
	 * permet de savoir si le jeu est en pause
	 * @return bool: true => le jeu est en pause
	 */
	bool isPaused();

	/**
	 * retourne le ModelDrawer utilis par le jeu.
	 * cette methode est utiliser pour partager le mme model drawer dans le jeu et dans le menu
	 * @return ModelDrawer: le modelDrawer.
	 */
	ModelDrawer * getDrawer();

	/**
	 * retourne le model de la loutre.
	 * comme pour le modelDrawer, cette methode est utilise pour partager le mme model de loutre
	 * entre le jeu et le menu
	 * @return Model: le model de la loutre.
	 */
	Model * getLoutreModel();

	/**
	 * retourne le niveau de difficult du jeu.
	 * cette methode est utiliser pour connaitre le niveau de difficult dans le menu de jeu.
	 * @return int: le niveau de difficult.
	 */
	int getDifficulty();

	/**
	 * dfinit le niveau de difficult.
	 * @param diff: le niveau de difficult.
	 */
	void setDifficulty(int diff);

	/**
	 * redemarre une nouvelle partie en reinitialisant les loutres, la position du joueur et les differents parametre
	 * de la partie.
	 */
	void startGame();

	/**
	 * methode du pattern <B>SINGLETON</B>, retourne l'instance unique de Game. si elle n'existe pas, la methode la cre.
	 * @return Game *: l'instance unique de Game.
	 */
	static Game * getGame();

	/**
	 * destructeur par defaut.
	 */
	virtual ~Game();

	/**
	 * methode appel par glut pour dessiner la scene.
	 */
	void draw(void);

	/**
	 * methode appele par glut lorsque la scene est fini d'etre dessine
	 */
	void idle(void);

	/**
	 * methode appele par glut lors d'un clic souris. Pour la definition des parametre voir les callback de glut
	 */
	void mouseClicked(int boutton,int etat,int x,int y);

	/**
	 * methode appele par glut lors d'un mouvement souris. Pour la definition des parametre voir les callback de glut
	 */
	void mouseMoved(int x,int y);

	/**
	 * methode appele par glut lors d'un mouvement avec clic de la souris. Pour la definition des parametre voir les callback de glut
	 */
	void mouseMovedClicked(int x,int y);

	/**
	 * methode appele par glut lors d'un appuye sur une touche. Pour la definition des parametre voir les callback de glut
	 */
	void keyPressed(unsigned char key,int x,int y);

	/**
	 * methode appele par glut lorsqu'une touche est relache. Pour la definition des parametre voir les callback de glut
	 */
	void keyReleased(unsigned char key,int x,int y);

	/**
	 * methode appele par glut lorsqu'une touche speciale est presse. Pour la definition des parametre voir les callback de glut
	 */
	void specialKeyFunc(int key,int x,int y);

	/**
	 * methode appele par glut lorsqu'une touche speciale est relache. Pour la definition des parametre voir les callback de glut
	 */
	void specialKeyUpFunc(int key,int x,int y);

	/**
	 * methode appele par glut lors du redimmentionnement de la fenetre. Pour la definition des parametre voir les callback de glut
	 */
	void reshape(int largeur,int hauteur);

};

#endif
